# -*- coding: utf-8 -*-
"""Test suit for postalcodes_mexico."""
import postalcodes_mexico

import pytest


def test_places_with_one_place():
    # GIVEN a single, existing postal code
    CP = '01000'

    # WHEN using the places method
    places = postalcodes_mexico.places(CP)

    # THEN it returns a list with exactly one place
    assert isinstance(places, list)
    assert len(places) == 1

    # AND that place is San Ángel
    place = places[0]
    assert place[1] == 'San Ángel'


def test_places_are_named_tuples():
    # GIVEN a single, existing postal code
    CP = '01000'

    # WHEN using the places method
    my_places = postalcodes_mexico.places(CP)
    my_place = my_places[0]

    # THEN it returns a list of named tuples
    assert isinstance(my_places, list)
    assert isinstance(my_place, tuple)

    # AND all attributes can be called
    assert my_place.postal_code == CP
    assert my_place.place == 'San Ángel'
    assert my_place.place_type == 'Colonia'
    assert my_place.municipality == 'Álvaro Obregón'
    assert my_place.city == 'Ciudad de México'
    assert my_place.state == 'Ciudad de México'


def test_places_with_multiple_places():
    # GIVEN another known postal code
    CP = '01030'

    # WHEN using the places method
    places = postalcodes_mexico.places(CP)

    # THEN it returns a list of two places
    assert isinstance(places, list)
    assert len(places) == 2


def test_postalcodes_without_parameter():
    # GIVEN the full database

    # WHEN calling the postalcodes method without a parameter
    cp_list = postalcodes_mexico.postalcodes()

    # THEN it returns a list of all Mexican postal codes
    assert isinstance(cp_list, list)
    assert len(cp_list) > 100000
    assert '01000' in cp_list


@pytest.mark.parametrize(
    'cp, cp_full', [
        ('01', '01000'),
        ('0103', '01030'),
        ('99', '99998'),
    ]
)
def test_postalcodes_with_beginning_of_postalcode(cp, cp_full):
    # GIVEN the beginning of a postal code

    # WHEN calling the postalcodes method with the beginning of the CP as a parameter
    cp_list = postalcodes_mexico.postalcodes(cp)

    # THEN a matching full postal code is in the returned list
    assert cp_full in cp_list


@pytest.mark.parametrize('cp', [('00'), ('99999'), ('88888'), ])
def test_non_existing_postalcodes(cp):
    # GIVEN the beginning of a non-existent postal code

    # WHEN calling the postalcodes method with it
    cp_list = postalcodes_mexico.postalcodes(cp)

    # THEN the returned list is empty
    assert len(cp_list) == 0



# -*- coding: utf-8 -*-
"""Test suit for postalcodes_mexico."""
import sqlite3

import postalcodes_mexico

import utils


def test_update_db():
    # GIVEN a test database and a short XML file
    test_db = 'test.db'
    xml_file = 'data/ciudad_de_mexico.xml'

    # WHEN calling the update method
    utils.update_db(db=test_db, xml_file=xml_file)

    con = sqlite3.connect(test_db)
    con.row_factory = sqlite3.Row

    # THEN the test database has entries
    CP = '01000'
    places = postalcodes_mexico.places(CP, db=test_db)

    assert len(places) == 1
    assert isinstance(places, list)

    place = places[0]
    assert place[0] == '01000'
    assert place[1] == 'San Ángel'
    assert place[2] == 'Colonia'
    assert place[3] == 'Álvaro Obregón'
    assert place[4] == 'Ciudad de México'
    assert place[5] == 'Ciudad de México'


def test_xmltolist():
    # GIVEN a short XML file
    xml_file = 'data/ciudad_de_mexico.xml'

    # WHEN calling the xmltolist method
    result = utils.xmltolist(xml_file=xml_file)

    # THEN it returns a list with a lot of places
    assert isinstance(result, list)
    assert len(result) > 1000

    # AND they are tuples
    assert isinstance(result[5], tuple)



# -*- coding: utf-8 -*-
"""Main package of 'postalcodes_mexico'."""

__author__ = """Florian Posdziech"""
__email__ = 'hallo@flowfx.de'
__version__ = '0.1.0'

import os
import sqlite3

from collections import namedtuple

BASE_DIR = os.path.dirname(os.path.realpath(__file__))
DB_PATH = os.path.abspath(os.path.join(BASE_DIR, 'postalcodes.sql'))


Location = namedtuple('Location', ['postal_code', 'place', 'place_type', 'municipality', 'city', 'state'])


def places(postalcode, db=DB_PATH):
    """Return a list of tuples with address information."""
    con = sqlite3.connect(db)
    con.row_factory = sqlite3.Row

    cp = (postalcode, )

    with con:
        c = con.cursor()

        c.execute('SELECT cp, place, place_type, municipality, city, state FROM places WHERE cp =?', cp)

        # create a list of named tuples
        # cf. https://docs.python.org/3/library/collections.html#collections.namedtuple
        result = [location for location in map(Location._make, c.fetchall())]

    return result


def postalcodes(postalcode='', db=DB_PATH):
    """Return a list of postal codes."""
    con = sqlite3.connect(db)
    con.row_factory = sqlite3.Row

    with con:
        c = con.cursor()

        c.execute('SELECT cp FROM places WHERE cp LIKE ?', ('{}%'.format(postalcode),))
        # cursor.execute("SELECT * FROM posts WHERE tags LIKE ?", ('%{}%'.format(tag),))

        result = [row[0] for row in c]

    return result
